/**
  ******************************************************************************
  * @file    Multi/Examples/MotionControl/IHM01A1_ExampleForRegisterHandling/Src/main.c 
  * @author  IPC Rennes
  * @version V1.5.0
  * @date    November 12, 2014
  * @brief   This example shows how handle L6474 registers
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT(c) 2014 STMicroelectronics</center></h2>
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
    
/** @defgroup IHM01A1_Example_for_RegisterHandling
  * @{
  */ 

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
 static volatile uint16_t gLastError;

/* Private function prototypes -----------------------------------------------*/
static void MyFlagInterruptHandler(void);

/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Main program
  * @param  None
  * @retval None
  */
int main(void)
{
  uint32_t registerValue;

  /* STM32xx HAL library initialization */
  HAL_Init();
  
  /* Configure the system clock */
  SystemClock_Config();
    
//----- Init of the L6474 library 
  /* Start the L6474 to use 1 device */
  /* The L6474 registers are set with the predefined values */
  /* from file l6474_target_config.h*/
  BSP_MotorControl_Init(BSP_MOTOR_CONTROL_BOARD_ID_L6474, 1);
  
  /* Attach the function MyFlagInterruptHandler (defined below) to the flag interrupt */
  BSP_MotorControl_AttachFlagInterrupt(MyFlagInterruptHandler);

  /* Attach the function Error_Handler (defined below) to the error Handler*/
  BSP_MotorControl_AttachErrorHandler(Error_Handler);
  
  /* Read ABS_POS register of device 0*/  
  registerValue = BSP_MotorControl_CmdGetParam(0, L6474_ABS_POS);

  if (registerValue != 1)
  {
    /* Set  ABS_POS register of device 0 to 1 */
    BSP_MotorControl_CmdSetParam(0, L6474_ABS_POS, 1);
  }
  
  /* Read EL_POS register of device 0*/  
  registerValue = BSP_MotorControl_CmdGetParam(0, L6474_EL_POS);

  if (registerValue != 0)
  {
    /* Set  EL_POS register of device 0 to 1 */
    BSP_MotorControl_CmdSetParam(0, L6474_EL_POS, 1);
  }

  /* Read MARK register of device 0*/  
  registerValue = BSP_MotorControl_CmdGetParam(0, L6474_MARK);

  if (registerValue != 1)
  {
    /* Set  MARK register of device 0 to 0 */
    BSP_MotorControl_CmdSetParam(0, L6474_MARK, 1);
  }

  /* Read T_FAST register of device 0*/  
  registerValue = BSP_MotorControl_CmdGetParam(0,L6474_T_FAST);  

  if (registerValue != ((uint8_t)L6474_CONF_PARAM_TOFF_FAST_DEVICE_1 |
                        (uint8_t)L6474_CONF_PARAM_FAST_STEP_DEVICE_1))
  {
    /* Set  T_FAST register of device 0 to predefined value of device 1 */
    BSP_MotorControl_CmdSetParam(0, L6474_T_FAST, (uint8_t)L6474_CONF_PARAM_TOFF_FAST_DEVICE_1 |(uint8_t)L6474_CONF_PARAM_FAST_STEP_DEVICE_1);
  }


  /* Read OCD_TH register of device 0*/  
  registerValue = BSP_MotorControl_CmdGetParam(0, L6474_OCD_TH);  

  if (registerValue != L6474_CONF_PARAM_OCD_TH_DEVICE_1)
  {
   /* Set  OCD_TH register of device 0 to predefined value of device 1 */
    BSP_MotorControl_CmdSetParam(0, L6474_OCD_TH, L6474_CONF_PARAM_OCD_TH_DEVICE_1);
  }

  /* Read STEP_MODE register of device 0*/  
  registerValue = BSP_MotorControl_CmdGetParam(0,L6474_STEP_MODE);    

  if (registerValue != ((uint8_t)L6474_CONF_PARAM_STEP_SEL_DEVICE_1 |
                        (uint8_t)L6474_CONF_PARAM_SYNC_SEL_DEVICE_1))
  {
   /* Set  STEP_MODE register of device 0 to predefined value of device 1*/
    BSP_MotorControl_CmdSetParam(0, L6474_STEP_MODE, (uint8_t)L6474_CONF_PARAM_STEP_SEL_DEVICE_1 |
                                                          (uint8_t)L6474_CONF_PARAM_SYNC_SEL_DEVICE_1);
  }

  /* Read ALARM_EN register of device 0*/  
  registerValue = BSP_MotorControl_CmdGetParam(0, L6474_ALARM_EN);    

  if (registerValue != L6474_CONF_PARAM_ALARM_EN_DEVICE_1)
  {
   /* Set ALARM_EN register of device 0 to predefined value of device 1 */
    BSP_MotorControl_CmdSetParam(0, L6474_ALARM_EN, L6474_CONF_PARAM_ALARM_EN_DEVICE_1);
  }

  /* Read CONFIG register of device 0*/  
  registerValue = BSP_MotorControl_CmdGetParam(0, L6474_CONFIG);    

  if (registerValue != ((uint16_t)L6474_CONF_PARAM_CLOCK_SETTING_DEVICE_1 |
                        (uint16_t)L6474_CONF_PARAM_TQ_REG_DEVICE_1 |
                        (uint16_t)L6474_CONF_PARAM_OC_SD_DEVICE_1 |
                        (uint16_t)L6474_CONF_PARAM_SR_DEVICE_1 |
                        (uint16_t)L6474_CONF_PARAM_TOFF_DEVICE_1))
  {
     /* Set CONFIG register of device 0 to predefined value of device 1 */
     BSP_MotorControl_CmdSetParam(0, L6474_CONFIG, (uint16_t)L6474_CONF_PARAM_CLOCK_SETTING_DEVICE_1 |
                                              (uint16_t)L6474_CONF_PARAM_TQ_REG_DEVICE_1 |
                                              (uint16_t)L6474_CONF_PARAM_OC_SD_DEVICE_1 |
                                              (uint16_t)L6474_CONF_PARAM_SR_DEVICE_1 |
                                              (uint16_t)L6474_CONF_PARAM_TOFF_DEVICE_1);
  
  }
  
  /* Infinite loop */
  while(1)
  {
    /* Request device 0 to go position -6400 */
    BSP_MotorControl_GoTo(0,-6400);
  
    /* Wait for the motor of device 0 ends moving */
    BSP_MotorControl_WaitWhileActive(0);
  
    /* Request device 0 to go position 6400 */
    BSP_MotorControl_GoTo(0,6400);
  
    /* Wait for the motor of device 0 ends moving */
    BSP_MotorControl_WaitWhileActive(0);  
  }
}

/**
  * @brief  This function is the User handler for the flag interrupt
  * @param  None
  * @retval None
  */
void MyFlagInterruptHandler(void)
{
  /* Get the value of the status register via the L6474 command GET_STATUS */
  uint16_t statusRegister = BSP_MotorControl_CmdGetStatus(0);
  
  /* Check HIZ flag: if set, power brigdes are disabled */
  if ((statusRegister & L6474_STATUS_HIZ) == L6474_STATUS_HIZ)
  {
    // HIZ state
    // Action to be customized            
  }

  /* Check direction bit */
  if ((statusRegister & L6474_STATUS_DIR) == L6474_STATUS_DIR)
  {
    // Forward direction is set
    // Action to be customized            
  }  
  else
  {
    // Backward direction is set
    // Action to be customized            
  }  

  /* Check NOTPERF_CMD flag: if set, the command received by SPI can't be performed */
  /* This often occures when a command is sent to the L6474 */
  /* while it is in HIZ state */
  if ((statusRegister & L6474_STATUS_NOTPERF_CMD) == L6474_STATUS_NOTPERF_CMD)
  {
       // Command received by SPI can't be performed
     // Action to be customized            
  }  

  /* Check WRONG_CMD flag: if set, the command does not exist */
  if ((statusRegister & L6474_STATUS_WRONG_CMD) == L6474_STATUS_WRONG_CMD)
  {
     //command received by SPI does not exist 
     // Action to be customized          
  }  

  /* Check UVLO flag: if not set, there is an undervoltage lock-out */
  if ((statusRegister & L6474_STATUS_UVLO) == 0)
  {
     //undervoltage lock-out 
     // Action to be customized          
  }  

  /* Check TH_WRN flag: if not set, the thermal warning threshold is reached */
  if ((statusRegister & L6474_STATUS_TH_WRN) == 0)
  {
    //thermal warning threshold is reached
    // Action to be customized          
  }    

  /* Check TH_SHD flag: if not set, the thermal shut down threshold is reached */
  if ((statusRegister & L6474_STATUS_TH_SD) == 0)
  {
    //thermal shut down threshold is reached 
    // Action to be customized          
  }    

  /* Check OCD  flag: if not set, there is an overcurrent detection */
  if ((statusRegister & L6474_STATUS_OCD) == 0)
  {
    //overcurrent detection 
    // Action to be customized          
  }      
 
}

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  error number of the error
  * @retval None
  */
void Error_Handler(uint16_t error)
{
  /* Backup error number */
  gLastError = error;
  
  /* Infinite loop */
  while(1)
  {
  }
}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif

/**
  * @}
  */


/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
